#include <iostream>
using namespace std;

template<size_t ... hodnoty>
struct posloupnost_hodnot {};

using przdn = posloupnost_hodnot<>;

template<size_t N, size_t...hodnoty>
struct n_t_hodnota;

template<size_t N, size_t h1, size_t...hodnoty>
struct n_t_hodnota<N, h1, hodnoty...>
{
	static constexpr auto vsledek = n_t_hodnota<N - 1, hodnoty...>::vsledek;
};

template<size_t h1, size_t ... hodnoty>
struct n_t_hodnota<0, h1, hodnoty...>
{
	static constexpr size_t vsledek = h1;
};

template<size_t N, size_t...hodnoty>
constexpr size_t poloka_posloupnosti(posloupnost_hodnot<hodnoty...>)
{
	static_assert(N < sizeof...(hodnoty), "Parametr N mus bt men ne dlka posloupnosti");
	return n_t_hodnota<N, hodnoty...>::vsledek;
}

template<size_t ... hodnoty>
void tisk1(posloupnost_hodnot<hodnoty...>)
{
	cout << "{";
	if constexpr (sizeof...(hodnoty) > 0)
	{
		size_t A[]{ hodnoty... };
		cout << A[0];
		for (int i = 1; i < sizeof...(hodnoty); i++)
			cout << ", " << A[i];
	}
	cout << "}";
}

template<size_t h1, size_t ... hodnoty>
void tisk_pomoc(ostream& proud, posloupnost_hodnot<h1, hodnoty...>)
{
	proud << h1;
	if constexpr (sizeof...(hodnoty) > 0)
	{
		((proud << ", " << hodnoty), ...);
	}
}

template<size_t ... hodnoty>
void tisk_impl(ostream& proud, posloupnost_hodnot<hodnoty...> p)
{
	proud << "{";
	if constexpr (sizeof...(hodnoty) > 0)
	{
		tisk_pomoc(proud, p);
	}
	proud << "}";
}

template<size_t ... hodnoty>
ostream& operator<<(ostream& proud, posloupnost_hodnot<hodnoty...> p)
{
	tisk_impl(proud, p);
	return proud;
}

//template<size_t...I, size_t ... J>
//constexpr auto spoj_posloupnosti(posloupnost_hodnot<I...>, posloupnost_hodnot<J...>) 
//                                   -> posloupnost_hodnot<I..., J...>;

template<size_t...I, size_t ... J>
constexpr auto spoj_posloupnosti(posloupnost_hodnot<I...>, posloupnost_hodnot<J...>)
-> posloupnost_hodnot<I..., J...>
{
	return posloupnost_hodnot<I..., J...>{};
}
     

template<class T, class U>
using spojen = decltype(spoj_posloupnosti(T{}, U{}));


int main()
{
	posloupnost_hodnot<1, 3, 5, 7, 9> p1;
	auto x = poloka_posloupnosti<2>(p1);
	tisk1(p1);
	cout << endl;
	cout << p1 << endl;
	posloupnost_hodnot<11, 13, 15> p2;
	auto p3 = spojen<decltype(p1), decltype(p2)>{};
	cout << p3 << endl;
	auto p4 = spoj_posloupnosti(p1, p2);
	cout << p4 << endl;
}